
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pspsdk.h>
#include <pspkernel.h>
#include <pspkerneltypes.h>
#include <pspuser.h>
#include <pspfpu.h>

#if !defined(MODULE_NAME)
	#define MODULE_NAME "VFPU test"
#endif


#if !defined(MODULE_VERSION_MAJOR)
	#define MODULE_VERSION_MAJOR 1
#endif


#if !defined(MODULE_VERSION_MINOR)
	#define MODULE_VERSION_MINOR 0
#endif


#if !defined(MODULE_ATTR)
	#define MODULE_ATTR 0
#endif


#define __stringify(s)	__tostring(s)
#define __tostring(s)	#s

PSP_HEAP_SIZE_MAX();

PSP_MODULE_INFO(__stringify(MODULE_NAME), MODULE_ATTR, MODULE_VERSION_MAJOR, MODULE_VERSION_MINOR);
PSP_MAIN_THREAD_ATTR(PSP_THREAD_ATTR_USER | THREAD_ATTR_VFPU);

static void setup_callbacks (void) __attribute__((constructor));
static void back_to_kernel (void) __attribute__((destructor));
static int update_thread (SceSize args, void *argp);
static int exit_callback (int arg1, int arg2, void *common);

static void back_to_kernel (void) {
	sceKernelExitGame();
}
static void setup_callbacks (void) {
	int id;
	if ((id = sceKernelCreateThread("update_thread", update_thread, 0x11, 0xFA0, 0, 0)) >= 0)
		sceKernelStartThread(id, 0, 0);
}
static int exit_callback (int arg1, int arg2, void *common) {
	sceKernelExitGame();
	return 0;
}

static int update_thread (SceSize args, void *argp) {
	int cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);
	sceKernelSleepThreadCB();
	return 0;
}

int ref_value = 0xDEADBEEF;

// Load to all fpu registers the same value
void registers_init() {
	__asm__ volatile (
	".set push\n"
	".set noreorder\n"
	"lwc1	$f0, %0\n"
	"lwc1	$f1, %0\n"
	"lwc1	$f2, %0\n"
	"lwc1	$f3, %0\n"
	"lwc1	$f4, %0\n"
	"lwc1	$f5, %0\n"
	"lwc1	$f6, %0\n"
	"lwc1	$f7, %0\n"
	"lwc1	$f8, %0\n"
	"lwc1	$f9, %0\n"
	"lwc1	$f10, %0\n"
	"lwc1	$f11, %0\n"
	"lwc1	$f12, %0\n"
	"lwc1	$f13, %0\n"
	"lwc1	$f14, %0\n"
	"lwc1	$f15, %0\n"
	"lwc1	$f16, %0\n"
	"lwc1	$f17, %0\n"
	"lwc1	$f18, %0\n"
	"lwc1	$f19, %0\n"
	"lwc1	$f20, %0\n"
	"lwc1	$f21, %0\n"
	"lwc1	$f22, %0\n"
	"lwc1	$f23, %0\n"
	"lwc1	$f24, %0\n"
	"lwc1	$f25, %0\n"
	"lwc1	$f26, %0\n"
	"lwc1	$f27, %0\n"
	"lwc1	$f28, %0\n"
	"lwc1	$f29, %0\n"
	"lwc1	$f30, %0\n"
	"lwc1	$f31, %0\n"
	".set   pop\n"
	::"m"(ref_value));
}

// Check any changes to registers
int registers_check() {
	int temp_array[32];
	__asm__ volatile (
	".set push\n"
	".set noreorder\n"
	"swc1	$f0, %0\n"
	"swc1	$f1, 4+%0\n"
	"swc1	$f2, 8+%0\n"
	"swc1	$f3, 12+%0\n"
	"swc1	$f4, 16+%0\n"
	"swc1	$f5, 20+%0\n"
	"swc1	$f6, 24+%0\n"
	"swc1	$f7, 28+%0\n"
	"swc1	$f8, 32+%0\n"
	"swc1	$f9, 36+%0\n"
	"swc1	$f10, 40+%0\n"
	"swc1	$f11, 44+%0\n"
	"swc1	$f12, 48+%0\n"
	"swc1	$f13, 52+%0\n"
	"swc1	$f14, 56+%0\n"
	"swc1	$f15, 60+%0\n"
	"swc1	$f16, 64+%0\n"
	"swc1	$f17, 68+%0\n"
	"swc1	$f18, 72+%0\n"
	"swc1	$f19, 76+%0\n"
	"swc1	$f20, 80+%0\n"
	"swc1	$f21, 84+%0\n"
	"swc1	$f22, 88+%0\n"
	"swc1	$f23, 92+%0\n"
	"swc1	$f24, 96+%0\n"
	"swc1	$f25, 100+%0\n"
	"swc1	$f26, 104+%0\n"
	"swc1	$f27, 108+%0\n"
	"swc1	$f28, 112+%0\n"
	"swc1	$f29, 116+%0\n"
	"swc1	$f30, 120+%0\n"
	"swc1	$f31, 124+%0\n"
	".set   pop\n"
	::"m"(temp_array[0]));

	int i;
	int ret = 0;
	for (i = 0; i < 32; i++) {
		if (temp_array[i] != ref_value) {
			ret |= (1 << i);
		}
	}
	return ret;
}

void ResultPrint(const char * result, int reg, FILE * fileo) {
	fwrite (result,1,strlen(result),fileo);
	char tempbuf[64];
	sprintf (tempbuf,"%x\n",reg);
	fwrite (tempbuf,1,strlen(tempbuf),fileo);
}

int main(int argc, char ** argv) {
	pspDebugScreenInit();
	pspDebugScreenSetBackColor(0x00000000);
	pspDebugScreenSetTextColor(0xFFFFFFFF);
	pspDebugScreenClear();

	pspDebugScreenPrintf("Testing registers...\n");

	FILE * fileo = fopen("results.log","w");

	float temp_register = 3.1416f;

	registers_init();
	__asm__ volatile ("ulv.q C000,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C000\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C010,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C010\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C020,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C020\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C030,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C030\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q C100,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C100\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C110,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C110\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C120,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C120\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C130,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C130\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q C200,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C200\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C210,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C210\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C220,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C220\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C230,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C230\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q C300,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C300\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C310,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C310\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C320,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C320\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C330,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C330\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q C400,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C400\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C410,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C410\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C420,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C420\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C430,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C430\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q C500,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C500\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C510,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C510\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C520,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C520\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C530,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C530\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q C600,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C600\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C610,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C610\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C620,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C620\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q C630,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q C630\n",registers_check(),fileo);


	// ROWS

	registers_init();
	__asm__ volatile ("ulv.q R000,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R000\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R001,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R001\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R002,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R002\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R003,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R003\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q R100,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R100\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R101,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R101\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R102,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R102\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R103,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R103\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q R200,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R200\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R201,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R201\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R202,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R202\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R203,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R203\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q R300,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R300\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R301,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R301\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R302,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R302\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R303,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R303\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q R400,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R400\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R401,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R401\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R402,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R402\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R403,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R403\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q R500,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R500\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R501,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R501\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R502,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R502\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R503,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R503\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("ulv.q R600,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R600\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R601,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R601\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R602,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R602\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("ulv.q R603,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("ulv.q R603\n",registers_check(),fileo);



////////////// SAVE


	registers_init();
	__asm__ volatile ("usv.q C000,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C000\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C010,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C010\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C020,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C020\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C030,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C030\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q C100,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C100\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C110,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C110\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C120,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C120\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C130,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C130\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q C200,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C200\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C210,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C210\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C220,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C220\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C230,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C230\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q C300,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C300\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C310,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C310\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C320,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C320\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C330,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C330\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q C400,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C400\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C410,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C410\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C420,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C420\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C430,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C430\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q C500,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C500\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C510,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C510\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C520,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C520\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C530,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C530\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q C600,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C600\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C610,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C610\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C620,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C620\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q C630,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q C630\n",registers_check(),fileo);


	// ROWS

	registers_init();
	__asm__ volatile ("usv.q R000,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R000\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R001,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R001\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R002,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R002\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R003,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R003\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q R100,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R100\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R101,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R101\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R102,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R102\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R103,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R103\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q R200,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R200\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R201,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R201\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R202,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R202\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R203,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R203\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q R300,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R300\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R301,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R301\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R302,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R302\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R303,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R303\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q R400,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R400\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R401,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R401\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R402,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R402\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R403,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R403\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q R500,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R500\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R501,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R501\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R502,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R502\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R503,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R503\n",registers_check(),fileo);

	registers_init();
	__asm__ volatile ("usv.q R600,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R600\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R601,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R601\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R602,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R602\n",registers_check(),fileo);
	registers_init();
	__asm__ volatile ("usv.q R603,%0\n"::"m"(temp_register));
	if (registers_check() != 0) ResultPrint("usv.q R603\n",registers_check(),fileo);

	pspDebugScreenPrintf ("Test done! All results saved in results.log");

	fclose(fileo);

	sceKernelDelayThreadCB(100000000);

	exit(0);
	return 0;
}


